//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <vector>
#include <string.h>
#include <stdio.h>
#include "ReadTriggers.h"
#include "Triggers.h"

using namespace std;

int main (int argc, char* argv[]){

  // check the argument
  if(argc!=20){
    cerr<<"usage:"<<endl;
    cerr<<argv[0]<<" [trigger pattern] [trigger selection] [printing selection]"<<endl; 
    cerr<<"[trigger pattern]:   full path to trigger files"<<endl; 
    cerr<<"[trigger selection]: this is a sequence of 6 arguments:"<<endl; 
    cerr<<"                     1) gps min"<<endl;
    cerr<<"                     2) gps max"<<endl;
    cerr<<"                     3) snr min"<<endl;
    cerr<<"                     4) snr max"<<endl;
    cerr<<"                     5) frequency min"<<endl;
    cerr<<"                     6) frequency max"<<endl;
    cerr<<"[printing selection]: this is a sequence of 12 arguments (1/0):"<<endl; 
    cerr<<"                     1) print frequency column"<<endl;
    cerr<<"                     2) print duration column"<<endl;
    cerr<<"                     3) print bandwidth column"<<endl;
    cerr<<"                     4) print tstart column"<<endl;
    cerr<<"                     5) print tend column"<<endl;
    cerr<<"                     6) print fstart column"<<endl;
    cerr<<"                     7) print fend column"<<endl;
    cerr<<"                     8) print snr column"<<endl;
    cerr<<"                     9) print amplitude column"<<endl;
    cerr<<"                    10) print q column"<<endl;
    cerr<<"                    11) print phase column"<<endl;
    cerr<<"                    12) clustering dt value"<<endl;
    cerr<<""<<endl;
    return 1;
  }

  // arguments
  string infiles = (string)argv[1];
  double tmin    =  atof(argv[2]);
  double tmax    =  atof(argv[3]);
  double snrmin  =  atof(argv[4]);
  double snrmax  =  atof(argv[5]);
  double freqmin =  atof(argv[6]);
  double freqmax =  atof(argv[7]);
  bool fprint    =  (bool)atoi(argv[8]);
  bool dprint    =  (bool)atoi(argv[9]);
  bool bprint    =  (bool)atoi(argv[10]);
  bool tsprint   =  (bool)atoi(argv[11]);
  bool teprint   =  (bool)atoi(argv[12]);
  bool fsprint   =  (bool)atoi(argv[13]);
  bool feprint   =  (bool)atoi(argv[14]);
  bool sprint    =  (bool)atoi(argv[15]);
  bool aprint    =  (bool)atoi(argv[16]);
  bool qprint    =  (bool)atoi(argv[17]);
  bool pprint    =  (bool)atoi(argv[18]);
  double cl_dt   =  atof(argv[19]);

  // input segments
  ReadTriggerSegments *insegments = new ReadTriggerSegments(infiles,"",0);
  if(!insegments->GetNFiles()){
    delete insegments;
    return 1;
  }
  
  // select files of interest
  string trigfiles = insegments->GetTriggerFiles(tmin,tmax);
  if(!trigfiles.compare("none")){
    delete insegments;
    return 1;
  }
  delete insegments;

  // input triggers
  ReadTriggers *intriggers;
  if(trigfiles.size()<10000) intriggers = new ReadTriggers(trigfiles,"",0);
  else intriggers = new ReadTriggers(infiles,"",0);
  if(!intriggers->GetNFiles()){
    delete intriggers;
    return 1;
  }
  trigfiles.clear();
 
   // print triggers
  if(cl_dt<=0.0){

    // header + optimize speed
    cout<<"# UN-clustered triggers"<<endl;
    if(tsprint) cout<<"# starting time [GPS]"<<endl;
    cout<<"# central time [GPS]"<<endl;
    if(teprint) cout<<"# ending time [GPS]"<<endl;
    if(dprint)  cout<<"# duration [s]"<<endl;
    if(fsprint) cout<<"# starting frequency [Hz]"<<endl;
    if(fprint)  cout<<"# central frequency [Hz]"<<endl;
    if(feprint) cout<<"# ending frequency [Hz]"<<endl;
    if(bprint)  cout<<"# bandwidth [Hz]"<<endl;
    if(qprint)  cout<<"# Q [-]"<<endl;
    else        intriggers->SetTriggerBranchStatus("q",false);
    if(sprint)  cout<<"# SNR [-]"<<endl;
    if(aprint)  cout<<"# amplitude [Hz^-1/2]"<<endl;
    else        intriggers->SetTriggerBranchStatus("amplitude",false);
    if(pprint)  cout<<"# phase [rad]"<<endl;
    else        intriggers->SetTriggerBranchStatus("phase",false);
     
    for(int t=0; t<intriggers->GetNTrig(); t++){

      // trigger selection
      if(intriggers->GetTriggerTimeEnd(t)<tmin) continue;
      if(intriggers->GetTriggerTimeStart(t)>=tmax) break;
      if(intriggers->GetTriggerTime(t)<tmin) continue;
      if(intriggers->GetTriggerTime(t)>=tmax) continue;
      if(intriggers->GetTriggerSNR(t)<snrmin||intriggers->GetTriggerSNR(t)>=snrmax) continue;
      if(intriggers->GetTriggerFrequency(t)<freqmin||intriggers->GetTriggerFrequency(t)>=freqmax) continue;

      // print
      if(tsprint) cout<<fixed<<setprecision(4)<<intriggers->GetTriggerTimeStart(t)<<" ";
      cout<<fixed<<setprecision(4)<<intriggers->GetTriggerTime(t)<<" ";
      if(teprint) cout<<fixed<<setprecision(4)<<intriggers->GetTriggerTimeEnd(t)<<" ";
      if(dprint) cout<<fixed<<setprecision(4)<<intriggers->GetTriggerDuration(t)<<" ";
      if(fsprint) cout<<fixed<<setprecision(2)<<intriggers->GetTriggerFrequencyStart(t)<<" ";
      if(fprint) cout<<fixed<<setprecision(2)<<intriggers->GetTriggerFrequency(t)<<" ";
      if(feprint) cout<<fixed<<setprecision(2)<<intriggers->GetTriggerFrequencyEnd(t)<<" ";
      if(bprint) cout<<fixed<<setprecision(2)<<intriggers->GetTriggerBandWidth(t)<<" ";
      if(qprint) cout<<fixed<<setprecision(2)<<intriggers->GetTriggerQ(t)<<" ";
      if(sprint) cout<<fixed<<setprecision(2)<<intriggers->GetTriggerSNR(t)<<" ";
      if(aprint) cout<<scientific<<setprecision(4)<<intriggers->GetTriggerAmplitude(t)<<" ";
      if(pprint) cout<<scientific<<setprecision(4)<<intriggers->GetTriggerPhase(t)<<" ";
      cout<<endl;
    }
  }
  // print clusters
  else{

    // clusterize
    intriggers->SetClusterizeDt(cl_dt);
    if(!intriggers->Clusterize()){
      cerr<<"input triggers are not time-sorted"<<endl;
      return 1;
    }
    intriggers->SetClusterBranchStatus("firstentry",false);
    intriggers->SetClusterBranchStatus("size",false);
    intriggers->SetTriggerBranchStatus("*",false);

    // header + optimize speed
    cout<<"# time-clustered triggers, dt = "<<cl_dt<<"s";
    cout<<endl;
    if(tsprint) cout<<"# starting time [GPS]"<<endl;
    cout<<"# central time [GPS]"<<endl;
    if(teprint) cout<<"# ending time [GPS]"<<endl;
    if(dprint)  cout<<"# duration [s]"<<endl;
    if(fsprint) cout<<"# starting frequency [Hz]"<<endl;
    if(fprint)  cout<<"# central frequency [Hz]"<<endl;
    if(feprint) cout<<"# ending frequency [Hz]"<<endl;
    if(bprint)  cout<<"# bandwidth [Hz]"<<endl;
    if(qprint)  cout<<"# Q [-]"<<endl;
    else        intriggers->SetClusterBranchStatus("q",false);
    if(sprint)  cout<<"# SNR [-]"<<endl;
    if(aprint)  cout<<"# amplitude [Hz^-1/2]"<<endl;
    else        intriggers->SetClusterBranchStatus("amplitude",false);
    if(pprint)  cout<<"# phase [rad]"<<endl;
    else        intriggers->SetClusterBranchStatus("phase",false);

    for(int c=0; c<intriggers->GetNClusters(); c++){
      
      // cluster selection
      if(intriggers->GetClusterTimeEnd(c)<tmin) continue;
      if(intriggers->GetClusterTimeStart(c)>=tmax) break;
      if(intriggers->GetClusterTime(c)<tmin||intriggers->GetClusterTime(c)>=tmax) continue;
      if(intriggers->GetClusterSNR(c)<snrmin||intriggers->GetClusterSNR(c)>=snrmax) continue;
      if(intriggers->GetClusterFrequency(c)<freqmin||intriggers->GetClusterFrequency(c)>=freqmax) continue;
      
      // print
      if(tsprint) cout<<fixed<<setprecision(4)<<intriggers->GetClusterTimeStart(c)<<" ";
      cout<<fixed<<setprecision(4)<<intriggers->GetClusterTime(c)<<" ";
      if(teprint) cout<<fixed<<setprecision(4)<<intriggers->GetClusterTimeEnd(c)<<" ";
      if(dprint)  cout<<fixed<<setprecision(4)<<intriggers->GetClusterDuration(c)<<" ";
      if(fsprint) cout<<fixed<<setprecision(2)<<intriggers->GetClusterFrequencyStart(c)<<" ";
      if(fprint)  cout<<fixed<<setprecision(2)<<intriggers->GetClusterFrequency(c)<<" ";
      if(feprint) cout<<fixed<<setprecision(2)<<intriggers->GetClusterFrequencyEnd(c)<<" ";
      if(bprint)  cout<<fixed<<setprecision(2)<<intriggers->GetClusterBandWidth(c)<<" ";
      if(qprint)  cout<<fixed<<setprecision(2)<<intriggers->GetClusterQ(c)<<" ";
      if(sprint)  cout<<fixed<<setprecision(2)<<intriggers->GetClusterSNR(c)<<" ";
      if(aprint)  cout<<scientific<<setprecision(4)<<intriggers->GetClusterAmplitude(c)<<" ";
      if(pprint)  cout<<scientific<<setprecision(4)<<intriggers->GetClusterPhase(c)<<" ";
      cout<<endl;
    }
  }

  // cleaning
  delete intriggers;
  
  return 0;
}

